@@ -1,7 +1,10 @@
-# (X)Emacs mode: -*- cperl -*-
-
 package Term::ProgressBar;
 
+use strict;
+use warnings;
+
+our $VERSION = '2.18';
+
 #XXX TODO Redo original test with count=20
 #         Amount Output
 #         Amount Prefix/Suffix
@@ -13,18 +16,22 @@ package Term::ProgressBar;
 #         If name is wider than term, trim name
 #         Don't update progress bar on new?
 
-=head1	NAME
+=head1 NAME
 
 Term::ProgressBar - provide a progress meter on a standard terminal
 
-=head1	SYNOPSIS
+=head1 VERSION
 
-  use Term::ProgressBar;
+Version 2.18
 
-  $progress = Term::ProgressBar->new ({count => $count});
-  $progress->update ($so_far);
+=head1 SYNOPSIS
 
-=head1	DESCRIPTION
+    use Term::ProgressBar;
+
+    my $progress = Term::ProgressBar->new ({count => 10_000});
+    $progress->update(5_000);
+
+=head1 DESCRIPTION
 
 Term::ProgressBar provides a simple progress bar on the terminal, to let the
 user know that something is happening, roughly how much stuff has been done,
@@ -34,7 +41,7 @@ A typical use sets up the progress bar with a number of items to do, and then
 calls L<update|"update"> to update the bar whenever an item is processed.
 
 Often, this would involve updating the progress bar many times with no
-user-visible change.  To avoid uneccessary work, the update method returns a
+user-visible change.  To avoid unnecessary work, the update method returns a
 value, being the update value at which the user will next see a change.  By
 only calling update when the current value exceeds the next update value, the
 call overhead is reduced.
@@ -55,26 +62,24 @@ bar for the same width.
 
 =head2 A really simple use
 
-  #!/usr/bin/perl
+    #!/usr/bin/perl
 
-  use Term::ProgressBar 2.00;
+    use Term::ProgressBar 2.00;
+    use constant MAX => 100_000;
 
-  use constant MAX => 100_000;
+    my $progress = Term::ProgressBar->new(MAX);
 
-  my $progress = Term::ProgressBar->new(MAX);
+    for (0..MAX) {
+        my $is_power = 0;
+        for (my $i = 0; 2**$i <= $_; $i++) {
+            $is_power = 1 if 2**$i == $_;
+        }
 
-  for (0..MAX) {
-    my $is_power = 0;
-    for(my $i = 0; 2**$i <= $_; $i++) {
-      $is_power = 1
-        if 2**$i == $_;
+        if ($is_power) {
+            $progress->update($_);
+        }
     }
 
-    if ( $is_power ) {
-      $progress->update($_);
-    }
-  }
-
 see eg/simle_use.pl
 
 Here is a simple example.  The process considers all the numbers between 0 and
@@ -87,18 +92,17 @@ distribution set (it is not installed as part of the module).
 
 =head2 A smoother bar update
 
-  my $progress = Term::ProgressBar->new($max);
+    my $progress = Term::ProgressBar->new($max);
 
-  for (0..$max) {
-    my $is_power = 0;
-    for(my $i = 0; 2**$i <= $_; $i++) {
-      $is_power = 1
-        if 2**$i == $_;
+    for (0..$max) {
+        my $is_power = 0;
+        for (my $i = 0; 2**$i <= $_; $i++) {
+            $is_power = 1 if 2**$i == $_;
+        }
+
+        $progress->update($_)
     }
 
-    $progress->update($_)
-  }
-
 See eg/smooth_bar.pl
 
 This example calls update for each value considered.  This will result in a
@@ -113,22 +117,20 @@ distribution set (it is not installed as part of the module.
 
 =head2 A (much) more efficient update
 
-  my $progress = Term::ProgressBar->new({name => 'Powers', count => $max, remove => 1});
-  $progress->minor(0);
-  my $next_update = 0;
+    my $progress = Term::ProgressBar->new({name => 'Powers', count => $max, remove => 1});
+    $progress->minor(0);
+    my $next_update = 0;
 
-  for (0..$max) {
-    my $is_power = 0;
-    for(my $i = 0; 2**$i <= $_; $i++) {
-      $is_power = 1
-        if 2**$i == $_;
+    for (0..$max) {
+        my $is_power = 0;
+        for (my $i = 0; 2**$i <= $_; $i++) {
+            $is_power = 1 if 2**$i == $_;
+        }
+
+        $next_update = $progress->update($_) if $_ >= $next_update;
     }
 
-    $next_update = $progress->update($_)
-      if $_ >= $next_update;
-  }
-  $progress->update($max)
-    if $max >= $next_update;
+    $progress->update($max) if $max >= $next_update;
 
 This example does two things to improve efficiency: firstly, it uses the value
 returned by L<update|"update"> to only call it again when needed; secondly, it
@@ -145,21 +147,84 @@ progress bar from the terminal when done.
 The complete text of this example is in F<examples/powers3> in the
 distribution set (it is not installed as part of the module.
 
+=head2 When the maximum number of items is sometimes unknown
+
+Sometimes you may wish to use the progress bar when the number of items may or
+may not be known. One common example is when you write a script that can take
+input piped from the output of another command, and then pipe the output to yet
+another command. eg:
+
+  some_command --arg value | my_script.pl | some_other_command
+
+Or ...
+
+  my_script.pl input_file output_file
+
+This example shows how you can iterate over a file specified on the command line
+with the progress bar. Since the input file may be read from STDIN, the number
+of lines may not be known. Term::ProgressBar handles this by just taking '-1' as
+the count value and with no further changes to the code. By calling update
+with the same count value, you ensure the progress bar is removed afterwards.
+
+    my $input_file = shift;
+    my $output_file = shift;
+    my $in_fh = \*STDIN;
+    my $out_fh = \*STDOUT;
+    my $message_fh = \*STDERR;
+    my $num_lines = -1;
+
+    if (defined($input_file) and $input_file ne '-') {
+        open($in_fh, $input_file) or die "Couldn't open file, '$input_file': $!";
+        my $wc_output = `wc -l $input_file`;
+        chomp($wc_output);
+        $wc_output =~ /^\s*(\d+)(\D.*)?/ or die "Couldn't parse wc output: $wc_output";
+        $num_lines = $1;
+    }
+
+    if(defined($output_file)) {
+        !-f $output_file or die "Specified output file, '$output_file', already exists";
+        open($out_fh, '>', $output_file) or die "Couldn't open output file, '$output_file': $!";
+    }
+
+    my $progress = Term::ProgressBar->new({
+        name   => 'file processor',
+        count  => $num_lines,
+        remove => 1,
+        fh     => $message_fh,
+    });
+
+    while (my $line = <$in_fh>) {
+        chomp($line);
+        print $out_fh "I found a line: $line\n";
+        $progress->message("Found 10000!") if($line =~ /10000/);
+        $progress->update();
+    }
+
+    $progress->update($num_lines);
+
+    print $message_fh "Finished\n";
+
+When the file is defined explicitly, the progress bar displays the linewise
+progress through the file. Since the progress bar by default prints output to
+stderr, your scripts output to STDOUT will not be affected.
+
 =head2 Using Completion Time Estimation
 
-  my $progress = Term::ProgressBar->new({name  => 'Powers',
-                                         count => $max,
-                                         ETA   => 'linear', });
-  $progress->max_update_rate(1);
-  my $next_update = 0;
+    my $progress = Term::ProgressBar->new({
+        name  => 'Powers',
+        count => $max,
+        ETA   => 'linear',
+    });
+    $progress->max_update_rate(1);
+    my $next_update = 0;
 
-  for (0..$max) {
-    my $is_power = 0;
-    for(my $i = 0; 2**$i <= $_; $i++) {
-      if ( 2**$i == $_ ) {
-        $is_power = 1;
-        $progress->message(sprintf "Found %8d to be 2 ** %2d", $_, $i);
-      }
+    for (0..$max) {
+        my $is_power = 0;
+        for (my $i = 0; 2**$i <= $_; $i++) {
+        if ( 2**$i == $_ ) {
+            $is_power = 1;
+            $progress->message(sprintf "Found %8d to be 2 ** %2d", $_, $i);
+        }
     }
 
     $next_update = $progress->update($_)
@@ -181,39 +246,11 @@ distribution set (it is not installed as part of the module.
 
 =cut
 
-# ----------------------------------------------------------------------
-
-# Pragmas --------------------------
-
-use strict;
-
-# Inheritance ----------------------
-
-use base qw( Exporter );
-use vars '@EXPORT_OK';
-@EXPORT_OK = qw( $PACKAGE $VERSION );
-
-# Utility --------------------------
-
 use Carp                    qw( croak );
 use Class::MethodMaker 1.02 qw( );
 use Fatal                   qw( open sysopen close seek );
 use POSIX                   qw( ceil strftime );
 
-# ----------------------------------------------------------------------
-
-# CLASS METHODS --------------------------------------------------------
-
-# ----------------------------------
-# CLASS CONSTANTS
-# ----------------------------------
-
-=head1 CLASS CONSTANTS
-
-Z<>
-
-=cut
-
 use constant MINUTE => 60;
 use constant HOUR   => 60 * MINUTE;
 use constant DAY    => 24 * HOUR;
@@ -242,27 +279,13 @@ use constant DEFAULTS => {
                           term_width => undef,
                           term       => undef,
                           remove     => 0,
+                          silent     => 0,
                          };
 
 use constant ETA_TYPES => { map { $_ => 1 } qw( linear ) };
 
 use constant ALREADY_FINISHED => 'progress bar already finished';
 
-use constant DEBUG => 0;
-
-# -------------------------------------
-
-use vars qw($PACKAGE $VERSION);
-$PACKAGE = 'Term-ProgressBar';
-$VERSION = '2.12';
-
-# ----------------------------------
-# CLASS CONSTRUCTION
-# ----------------------------------
-
-# ----------------------------------
-# CLASS COMPONENTS
-# ----------------------------------
 
 # This is here to allow testing to redirect away from the terminal but still
 # see terminal output, IYSWIM
@@ -286,7 +309,8 @@ sub __force_term {
 # ----------------------------------
 
 sub term_size {
-  my ($fh) = @_;
+  my ( $self, $fh ) = @_;
+  return if $self->silent;
 
   eval {
     require Term::ReadKey;
@@ -298,7 +322,7 @@ sub term_size {
   my $result;
   eval {
     $result = (Term::ReadKey::GetTerminalSize($fh))[0];
-    $result-- if ($^O eq "MSWin32");
+    $result-- if ($^O eq "MSWin32" or $^O eq "cygwin");
   }; if ( $@ ) {
     warn "error from Term::ReadKey::GetTerminalSize(): $@";
   }
@@ -316,21 +340,10 @@ sub term_size {
   return $result;
 }
 
-
-# INSTANCE METHODS -----------------------------------------------------
-
-# ----------------------------------
-# INSTANCE CONSTRUCTION
-# ----------------------------------
+# Don't document hash keys until tested that the give the desired affect!
 
 =head1 INSTANCE CONSTRUCTION
 
-Z<>
-
-=cut
-
-# Don't document hash keys until tested that the give the desired affect!
-
 =head2 new
 
 Create & return a new Term::ProgressBar instance.
@@ -350,6 +363,12 @@ a number, being equivalent to the C<count> key.
 The item count.  The progress is marked at 100% when update I<count> is
 invoked, and proportionally until then.
 
+If you specify a count less than zero, just the name (if specified) will be
+displayed and (if the remove flag is set) removed when the progress bar is
+updated with a number lower than zero. This allows you to use the progress bar
+when the count is sometimes known and sometimes not without making multiple
+changes throughout your code.
+
 =item name
 
 A name to prefix the progress bar with.
@@ -360,6 +379,25 @@ The filehandle to output to.  Defaults to stderr.  Do not try to use
 *foo{THING} syntax if you want Term capabilities; it does not work.  Pass in a
 globref instead.
 
+=item term_width
+
+Sometimes we can't correctly determine the terminal width. You can use this
+parameter to force a term width of a particular size. Use a positive integer,
+please :)
+
+=item silent
+
+If passed a true value, Term::ProgressBar will do nothing at all. Useful in
+scripts where the progress bar is optional (or just plain doesn't work due to
+issues with modules it relies on).
+
+Instead, tell the constructor you want it to be silent and you don't need to
+change the rest of your program:
+
+    my $progress = Term::ProgressBar->new( { count => $count, silent => $silent } );
+    # later
+    $progress->update; # does nothing
+
 =item ETA
 
 A total time estimation to use.  If enabled, a time finished estimation is
@@ -425,13 +463,15 @@ Class::MethodMaker->import (new_with_init => 'new',
 
 sub init {
   my $self = shift;
+  return if $self->silent;
 
   # V1 Compatibility
   return $self->init({count      => $_[1], name => $_[0],
                       term_width => 50,    bar_width => 50,
                       major_char => '#',   minor_char => '',
                       lbrack     => '',    rbrack     => '',
-                      term       => '0 but true', })
+                      term       => '0 but true',
+                      silent     => 0,})
     if @_ == 2;
 
   my $target;
@@ -471,7 +511,7 @@ sub init {
     die "term width $config{term_width} (from __force_term) too small"
       if $config{term_width} < 5;
   } elsif ( $config{term} and ! defined $config{term_width}) {
-    $config{term_width} = term_size($config{fh});
+    $config{term_width} = $self->term_size($config{fh});
     die if $config{term_width} < 5;
   }
 
@@ -496,7 +536,7 @@ sub init {
     } else {
       $config{bar_width}  = $target;
       die "configured bar_width $config{bar_width} < 1"
- 	if $config{bar_width} < 1;
+      if $config{bar_width} < 1;
     }
   }
 
@@ -633,6 +673,7 @@ Class::MethodMaker->import
                          offset      scale
                          fh          start
                          max_update_rate
+                         silent
                      /],
    counter       => [qw/ last_position last_update /],
    boolean       => [qw/ minor name_printed pb_ended remove /],
@@ -643,6 +684,7 @@ Class::MethodMaker->import
 # We generate these by hand since we want to check the values.
 sub bar_width {
     my $self = shift;
+    return if $self->silent;
     return $self->{bar_width} if not @_;
     croak 'wrong number of arguments' if @_ != 1;
     croak 'bar_width < 1' if $_[0] < 1;
@@ -650,6 +692,7 @@ sub bar_width {
 }
 sub term_width {
     my $self = shift;
+    return if $self->silent;
     return $self->{term_width} if not @_;
     croak 'wrong number of arguments' if @_ != 1;
     croak 'term_width must be at least 5' if $self->term and $_[0] < 5;
@@ -658,6 +701,7 @@ sub term_width {
 
 sub target {
   my $self = shift;
+  return if $self->silent;
 
   if ( @_ ) {
     my ($target) = @_;
@@ -676,7 +720,7 @@ sub target {
 
 sub ETA {
   my $self = shift;
-
+  return if $self->silent;
   if (@_) {
     my ($type) = @_;
     croak "Invalid ETA type: $type\n"
@@ -744,6 +788,7 @@ The next value of so_far at which to call C<update>.
 
 sub update {
   my $self = shift;
+  return if $self->silent;
   my ($so_far) = @_;
 
   if ( ! defined $so_far ) {
@@ -759,12 +804,28 @@ sub update {
   my $name = $self->name;
   my $fh = $self->fh;
 
-  if ( $target < 1 ) {
+
+  if ( $target < 0 ) {
+    if($input_so_far <= 0 or $input_so_far == $self->last_update) {
+      print $fh "\r", ' ' x $self->term_width, "\r";
+
+      if(defined $name) {
+        if(!$self->remove or $input_so_far >= 0) {
+          print $fh "$name...";
+        }
+		if(!$self->remove and $input_so_far < 0) {
+		  print $fh "\n";
+        }
+      }
+    }
+    $self->last_update($input_so_far);
+    return 2**32-1;
+  } elsif ( $target == 0 ) {
     print $fh "\r";
     printf $fh "$name: "
       if defined $name;
     print $fh "(nothing to do)\n";
-    return 2**32-1;
+	return 2**32-1;
   }
 
   my $biggies     = $self->major_units * $so_far;
@@ -838,10 +899,10 @@ sub update {
       }
     }
     for ($self->{last_printed}) {
-	unless (defined and $_ eq $to_print) {
-	    print $fh $to_print;
-	}
-	$_ = $to_print;
+        unless (defined and $_ eq $to_print) {
+            print $fh $to_print;
+        }
+        $_ = $to_print;
     }
 
     $next -= $self->offset;
@@ -923,6 +984,7 @@ The message to output.
 
 sub message {
   my $self = shift;
+  return if $self->silent;
   my ($string) = @_;
   chomp ($string);
 
@@ -942,13 +1004,9 @@ sub message {
 
 # ----------------------------------------------------------------------
 
-=head1 BUGS
-
-Z<>
-
 =head1 REPORTING BUGS
 
-Email the author.
+via RT: L<https://rt.cpan.org/Dist/Display.html?Name=Term-ProgressBar>
 
 =head1 COMPATIBILITY
 
@@ -958,24 +1016,24 @@ Various other defaults are set to emulate version one (e.g., the major output
 character is '#', the bar width is set to 50 characters and the output
 filehandle is not treated as a terminal). This mode is deprecated.
 
-=head1	AUTHOR
+=head1 AUTHOR
 
 Martyn J. Pearce fluffy@cpan.org
 
 Significant contributions from Ed Avis, amongst others.
 
-=head1 COPYRIGHT
+=head1 MAINTAINER
+
+Gabor Szabo L<http://szabgab.com/> L<http://perlmaven.com/>
+
+=head1 LICENSE AND COPYRIGHT
 
 Copyright (c) 2001, 2002, 2003, 2004, 2005 Martyn J. Pearce.  This program is
 free software; you can redistribute it and/or modify it under the same terms
 as Perl itself.
 
-=head1	SEE ALSO
-
-Z<>
-
 =cut
 
-1; # keep require happy.
+1;
 
 __END__
